@@ -1,4 +1,6 @@
 /*
+ *  Copyright (c) 2016, Peter Haag
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
@@ -11,7 +13,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *     this list of conditions and the following disclaimer in the documentation 
  *     and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *     used to endorse or promote products derived from this software without 
  *     specific prior written permission.
  *  
@@ -27,16 +29,11 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: haag $
- *
- *  $Id: bookkeeper.c 39 2009-11-25 08:11:15Z haag $
- *
- *  $LastChangedRevision: 39 $
- *	
- *
  */
 
+#ifdef HAVE_CONFIG_H 
 #include "config.h"
+#endif
 
 #include <stdio.h>
 #include <unistd.h>
@@ -64,22 +61,14 @@ union semun {
 };
 #endif
 
-#include "config.h"
-
+#include "util.h"
 #include "bookkeeper.h"
 
 static bookkeeper_list_t *bookkeeper_list = NULL;
 
 /* function prototypes */
 
-/* 
- * bookkeeper.c is needed for daemon code as well as normal stdio code 
- * therefore a generic LogError is defined, which maps to the 
- * approriate logging channel - either stderr or syslog
- */
-void LogError(char *format, ...);
-
-static key_t _ftok(const char *path, int id);
+static uint32_t hash(char *str, int flag);
 
 static void sem_lock(int sem_set_id);
 
@@ -89,16 +78,21 @@ static inline bookkeeper_list_t *Get_bookkeeper_list_entry(bookkeeper_t *bookkee
 
 /* Create shared memory object and set its size */
 
-/* our own ftok implementation - the standard C library ftok is not reliable enough */
-static key_t _ftok(const char *path, int id) {
-struct stat st;
+/* hash: compute hash value of string */
+#define MULTIPLIER 37
+static uint32_t hash(char *str, int flag) {
+uint32_t h; 
+unsigned char *p;
 
-    if (stat(path, &st) < 0)
-        return (key_t)-1;
-
-    return (key_t) ( ((st.st_dev & 0xffff) << 16) ^ st.st_ino ) + id;
-}
+	h = 0;
+	for (p = (unsigned char*)str; *p != '\0'; p++)
+		h = MULTIPLIER * h + *p;
 
+	if ( flag ) {
+		h = MULTIPLIER * h + 'R';
+	}
+	return h; // or, h % ARRAY_SIZE;
+} // End of hash
 
 // locks the semaphore, for exclusive access to the bookkeeping record
 static void sem_lock(int sem_set_id) {
@@ -151,7 +145,7 @@ bookkeeper_list_t	**bookkeeper_list_entry;
 
 	*bookkeeper = NULL;
 
-	shm_key = _ftok(path, 1); 
+	shm_key = hash(path, 0); 
 	if ( shm_key == - 1 ) 
 		return ERR_PATHACCESS;
 
@@ -239,7 +233,7 @@ bookkeeper_list_t	**bookkeeper_list_entry;
 
 	// create semaphore
 
-	sem_key = _ftok(path, 2); 
+	sem_key = hash(path, 1); 
 	// this should never fail, as we aleady got a key for the shared memory
 	if ( sem_key == - 1 ) {
 		// .. but catch it anyway .. and release shared memory. something is fishy
@@ -309,7 +303,7 @@ int sem_key, shm_key, shm_id, sem_id;
 
 	*bookkeeper = NULL;
 
-	shm_key = _ftok(path, 1); 
+	shm_key = hash(path, 0); 
 	if ( shm_key == - 1 ) 
 		return ERR_PATHACCESS;
 
@@ -340,7 +334,7 @@ int sem_key, shm_key, shm_id, sem_id;
 	// at this point we now have a valid record and can proceed
 
 	// create semaphore
-	sem_key = _ftok(path, 2); 
+	sem_key = hash(path, 1); 
 	// this should never fail, as we aleady got a key for the shared memory
 	if ( sem_key == - 1 ) {
 		// .. but catch it anyway .. and release shared memory. something is fishy
