@@ -1,4 +1,6 @@
 /*
+ *  Copyright (c) 2017, Peter Haag
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
@@ -11,7 +13,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *     this list of conditions and the following disclaimer in the documentation 
  *     and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *     used to endorse or promote products derived from this software without 
  *     specific prior written permission.
  *  
@@ -27,17 +29,22 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: haag $
- *
- *  $Id: collector.h 51 2010-01-29 09:01:54Z haag $
- *
- *  $LastChangedRevision: 51 $
- *	
  */
 
 #ifndef _COLLECTOR_H
 #define _COLLECTOR_H 1
 
+#include "config.h"
+
+#include <sys/types.h>
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
+#include <sys/socket.h>
+
+#include "bookkeeper.h"
+#include "nffile.h"
+
 #define FNAME_SIZE  256
 #define IDENT_SIZE  32
 
@@ -52,12 +59,6 @@ typedef struct srecord_s {
 // common_record_t defines ext_map as uint_8, so max 256 extension maps allowed.
 // should be enough anyway
 
-typedef struct sampler_s {
-	uint32_t	interval;
-	uint16_t	table_id;
-	uint8_t		mode;
-} sampler_t;
-
 typedef struct option_offset_s {
 	struct option_offset_s *next;
 	uint32_t	id;					// table id
@@ -76,6 +77,26 @@ typedef struct option_offset_s {
 
 } option_offset_t;
 
+typedef struct generic_sampler_s {
+	struct generic_sampler_s *next;
+	sampler_info_record_t	info;
+} generic_sampler_t;
+
+typedef struct generic_exporter_s {
+	// link chain
+	struct generic_exporter_s *next;
+
+	// generic exporter information
+	exporter_info_record_t info;
+
+	uint64_t	packets;			// number of packets sent by this exporter
+	uint64_t	flows;				// number of flow records sent by this exporter
+	uint32_t	sequence_failure;	// number of sequence failues
+
+	generic_sampler_t		*sampler;
+
+} generic_exporter_t;
+
 typedef struct FlowSource_s {
 	// link
 	struct FlowSource_s *next;
@@ -98,29 +119,20 @@ typedef struct FlowSource_s {
 	uint64_t			first_seen;		// in msec 
 	uint64_t			last_seen;		// in msec
 
-	// port histogram data
-	xstat_t				*xstat;
-
 	// Any exporter specific data
-	void				*exporter_data;
+	generic_exporter_t	*exporter_data;
+	uint32_t			exporter_count;
+	struct timeval		received;
 
 	// extension map list
 	struct {
-#define MAP_BLOCKSIZE	16
+#define BLOCK_SIZE	16
 		int	next_free;
 		int	max_maps;
+		int num_maps;
 		extension_map_t	**maps;
 	} extension_map_list;
 
-	// Netflow v9 only:
-	// sampling information: 
-	// each flow source may have several sampler applied
-	// tags #48, #49, #50
-	sampler_t 		 **sampler;
-
-	// global sampling information #34 #35
-	sampler_t		std_sampling;
-
 	option_offset_t *option_offset_table;
 
 } FlowSource_t;
@@ -133,19 +145,25 @@ int AddFlowSource(FlowSource_t **FlowSource, char *ident);
 
 int AddDefaultFlowSource(FlowSource_t **FlowSource, char *ident, char *path);
 
+int SetDynamicSourcesDir(FlowSource_t **FlowSource, char *dir);
+
+FlowSource_t *AddDynamicSource(FlowSource_t **FlowSource, struct sockaddr_storage *ss);
+
 int InitExtensionMapList(FlowSource_t *fs);
 
+int ReInitExtensionMapList(FlowSource_t *fs);
+
+int RemoveExtensionMap(FlowSource_t *fs, extension_map_t *map);
+
 int AddExtensionMap(FlowSource_t *fs, extension_map_t *map);
 
-void FlushExtensionMaps(FlowSource_t *fs);
+void FlushStdRecords(FlowSource_t *fs);
 
-void InsertSamplerOffset( FlowSource_t *fs, uint16_t id, uint16_t offset_sampler_id, uint16_t sampler_id_length, 
-	uint16_t offset_sampler_mode, uint16_t offset_sampler_interval);
+void FlushExporterStats(FlowSource_t *fs);
 
-void InsertStdSamplerOffset( FlowSource_t *fs, uint16_t id, uint16_t offset_std_sampler_interval, 
-	uint16_t offset_std_sampler_algorithm);
+int FlushInfoExporter(FlowSource_t *fs, exporter_info_record_t *exporter);
 
-void InsertSampler( FlowSource_t *fs, uint8_t sampler_id, sampler_t *sampler);
+int FlushInfoSampler(FlowSource_t *fs, sampler_info_record_t *sampler);
 
 int HasOptionTable(FlowSource_t *fs, uint16_t id );
 
