@@ -1,4 +1,5 @@
 /*
+ *  Copyright (c) 2017, Peter Haag
  *  Copyright (c) 2016, Peter Haag
  *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2012, Peter Haag
@@ -61,7 +62,6 @@
 #include "nfx.h"
 #include "nfnet.h"
 #include "bookkeeper.h"
-#include "nfxstat.h"
 #include "collector.h"
 #include "exporter.h"
 #include "nf_common.h"
@@ -151,14 +151,14 @@ char *p1, *p2;
 		#define IP_STRING_LEN   40
 		char ipstr[IP_STRING_LEN];
 		if ( exporter_record->sa_family == AF_INET ) {
-			uint32_t _ip = htonl(exporter_record->ip.v4);
+			uint32_t _ip = htonl(exporter_record->ip.V4);
 			inet_ntop(AF_INET, &_ip, ipstr, sizeof(ipstr));
 			printf("SysID: %u, IP: %16s, version: %u, ID: %2u, Slot: %u\n", exporter_record->sysid,
 				ipstr, exporter_record->version, exporter_record->id, id);
 		} else if ( exporter_record->sa_family == AF_INET6 ) {
 			uint64_t _ip[2];
-			_ip[0] = htonll(exporter_record->ip.v6[0]);
-			_ip[1] = htonll(exporter_record->ip.v6[1]);
+			_ip[0] = htonll(exporter_record->ip.V6[0]);
+			_ip[1] = htonll(exporter_record->ip.V6[1]);
 			inet_ntop(AF_INET6, &_ip, ipstr, sizeof(ipstr));
 			printf("SysID: %u, IP: %40s, version: %u, ID: %2u, Slot: %u\n", exporter_record->sysid,
 				ipstr, exporter_record->version, exporter_record->id, id);
@@ -347,14 +347,14 @@ uint64_t total_bytes;
 					found = 1;
 					printf("\n");
 					if ( exporter_record->sa_family == AF_INET ) {
-						uint32_t _ip = htonl(exporter_record->ip.v4);
+						uint32_t _ip = htonl(exporter_record->ip.V4);
 						inet_ntop(AF_INET, &_ip, ipstr, sizeof(ipstr));
 						printf("SysID: %u, IP: %16s, version: %u, ID: %2u, Sequence Failures: %u\n", exporter_record->sysid,
 							ipstr, exporter_record->version, exporter_record->exporter_id, exporter_record->sequence_failure);
 					} else if ( exporter_record->sa_family == AF_INET6 ) {
 						uint64_t _ip[2];
-						_ip[0] = htonll(exporter_record->ip.v6[0]);
-						_ip[1] = htonll(exporter_record->ip.v6[1]);
+						_ip[0] = htonll(exporter_record->ip.V6[0]);
+						_ip[1] = htonll(exporter_record->ip.V6[1]);
 						inet_ntop(AF_INET6, &_ip, ipstr, sizeof(ipstr));
 						printf("SysID: %u, IP: %40s, version: %u, ID: %2u, Sequence Failures: %u\n", exporter_record->sysid,
 							ipstr, exporter_record->version, exporter_record->exporter_id, exporter_record->sequence_failure);
@@ -410,7 +410,7 @@ uint64_t total_bytes;
 		printf("\n");
 		exporter = &exporter_list[i]->info;
 		if ( exporter->sa_family == AF_INET ) {
-			uint32_t _ip = htonl(exporter->ip.v4);
+			uint32_t _ip = htonl(exporter->ip.V4);
 			inet_ntop(AF_INET, &_ip, ipstr, sizeof(ipstr));
 			if ( exporter_list[i]->flows ) 
 				printf("SysID: %u, IP: %16s, version: %u, ID: %2u, Sequence failures: %u, packets: %llu, flows: %llu\n", 
@@ -424,8 +424,8 @@ uint64_t total_bytes;
 					
 		} else if ( exporter->sa_family == AF_INET6 ) {
 			uint64_t _ip[2];
-			_ip[0] = htonll(exporter->ip.v6[0]);
-			_ip[1] = htonll(exporter->ip.v6[1]);
+			_ip[0] = htonll(exporter->ip.V6[0]);
+			_ip[1] = htonll(exporter->ip.V6[1]);
 			inet_ntop(AF_INET6, &_ip, ipstr, sizeof(ipstr));
 			if ( exporter_list[i]->flows ) 
 				printf("SysID: %u, IP: %40s, version: %u, ID: %2u, Sequence failures: %u, packets: %llu, flows: %llu\n ", 
