@@ -1,4 +1,6 @@
 /*
+ *  Copyright (c) 2017, Peter Haag
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
@@ -11,7 +13,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *     this list of conditions and the following disclaimer in the documentation 
  *     and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *     used to endorse or promote products derived from this software without 
  *     specific prior written permission.
  *  
@@ -27,12 +29,6 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: haag $
- *
- *  $Id: netflow_v9.h 51 2010-01-29 09:01:54Z haag $
- *
- *  $LastChangedRevision: 51 $
- *	
  */
 
 /* v9 structures */
@@ -71,6 +67,17 @@
 #ifndef _NETFLOW_V9_H
 #define _NETFLOW_V9_H 1
 
+#include "config.h"
+
+#include <sys/types.h>
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
+
+#include "collector.h"
+#include "nfnet.h"
+#include "nffile.h"
+
 typedef struct netflow_v9_header {
 	uint16_t  version;
 	uint16_t  count;
@@ -159,7 +166,12 @@ typedef struct common_header_s {
 #define _4bytes   4
 #define _6bytes   6
 #define _8bytes   8
+#define _12bytes  12
 #define _16bytes  16
+#define _20bytes  20
+#define _24bytes  24
+#define _65bytes  65
+#define _72bytes  72
 
 #define NF9_TEMPLATE_FLOWSET_ID     0
 #define NF9_OPTIONS_FLOWSET_ID      1
@@ -205,9 +217,13 @@ typedef struct common_header_s {
 #define NF9_ENGINE_ID			39
 
 #define NF9_FLOW_SAMPLER_ID 	48 
-#define FLOW_SAMPLER_MODE 	49 
+#define FLOW_SAMPLER_MODE 		49 
 #define NF9_FLOW_SAMPLER_RANDOM_INTERVAL 50
 
+#define NF_SELECTOR_ID			302
+#define NF_SELECTOR_ALGORITHM	304
+#define NF_SAMPLING_INTERVAL	305
+
 // #define NF9_MIN_TTL			52
 // #define NF9_MAX_TTL			53
 // #define NF9_IPV4_IDENT		54
@@ -242,6 +258,47 @@ typedef struct common_header_s {
 #define NF9_BGP_ADJ_NEXT_AS 	128
 #define NF9_BGP_ADJ_PREV_AS 	129
 
+// CISCO ASA NSEL extension - Network Security Event Logging
+#define NF_F_FLOW_BYTES				   85
+#define NF_F_CONN_ID                  148
+#define NF_F_FLOW_CREATE_TIME_MSEC	  152
+#define NF_F_FLOW_END_TIME_MSEC		  153
+#define NF_F_ICMP_TYPE                176
+#define NF_F_ICMP_CODE                177
+#define NF_F_ICMP_TYPE_IPV6           178
+#define NF_F_ICMP_CODE_IPV6           179
+#define NF_F_FWD_FLOW_DELTA_BYTES	  231
+#define NF_F_REV_FLOW_DELTA_BYTES	  232
+#define NF_F_EVENT_TIME_MSEC          323
+#define NF_F_INGRESS_ACL_ID         33000
+#define NF_F_EGRESS_ACL_ID          33001
+#define NF_F_FW_EXT_EVENT           33002
+#define NF_F_USERNAME               40000
+
+#define NF_F_XLATE_SRC_ADDR_IPV4	  225
+#define NF_F_XLATE_DST_ADDR_IPV4	  226
+#define NF_F_XLATE_SRC_PORT			  227
+#define NF_F_XLATE_DST_PORT			  228
+#define NF_F_XLATE_SRC_ADDR_IPV6	  281
+#define NF_F_XLATE_DST_ADDR_IPV6	  282
+#define NF_F_FW_EVENT				  233
+
+// ASA 8.4 compat elements
+#define NF_F_XLATE_SRC_ADDR_84		40001
+#define NF_F_XLATE_DST_ADDR_84		40002
+#define NF_F_XLATE_SRC_PORT_84      40003
+#define NF_F_XLATE_DST_PORT_84      40004
+#define NF_F_FW_EVENT_84            40005
+
+// Cisco ASR 1000 series NEL extension - Nat Event Logging
+#define NF_N_NAT_EVENT				230
+#define NF_N_INGRESS_VRFID			234
+#define NF_N_EGRESS_VRFID			235
+#define NF_F_XLATE_PORT_BLOCK_START 361
+#define NF_F_XLATE_PORT_BLOCK_END   362
+#define NF_F_XLATE_PORT_BLOCK_STEP  363
+#define NF_F_XLATE_PORT_BLOCK_SIZE  364
+
 // nprobe latency extensions
 #define NF9_NPROBE_CLIENT_NW_DELAY_SEC	57554
 #define NF9_NPROBE_CLIENT_NW_DELAY_USEC	57555
