@@ -1,4 +1,5 @@
 /*
+ *  Copyright (c) 2017, Peter Haag
  *  Copyright (c) 2016, Peter Haag
  *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
@@ -29,8 +30,6 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  Author: peter
- *
  */
 
 /*
@@ -86,7 +85,6 @@
 #include "flist.h"
 #include "nfstatfile.h"
 #include "bookkeeper.h"
-#include "nfxstat.h"
 #include "collector.h"
 #include "exporter.h"
 #include "netflow_v1.h"
@@ -149,7 +147,7 @@ static void daemonize(void);
 static void SetPriv(char *userid, char *groupid );
 
 static void run(packet_function_t receive_packet, int socket, send_peer_t peer, 
-	time_t twin, time_t t_begin, int report_seq, int use_subdirs, char *time_extension, int compress, int do_xstat);
+	time_t twin, time_t t_begin, int report_seq, int use_subdirs, char *time_extension, int compress);
 
 /* Functions */
 static void usage(char *name) {
@@ -171,8 +169,9 @@ static void usage(char *name) {
 					"-R IP[/port]\tRepeat incoming packets to IP address/port\n"
 					"-s rate\tset default sampling rate (default 1)\n"
 					"-x process\tlaunch process after a new file becomes available\n"
-					"-j\t\tBZ2 compress flows in output file.\n"
 					"-z\t\tLZO compress flows in output file.\n"
+					"-y\t\tLZ4 compress flows in output file.\n"
+					"-j\t\tBZ2 compress flows in output file.\n"
 					"-B bufflen\tSet socket buffer to bufflen bytes\n"
 					"-e\t\tExpire data at each cycle.\n"
 					"-D\t\tFork to background\n"
@@ -360,7 +359,7 @@ int		err;
 #include "collector_inline.c"
 
 static void run(packet_function_t receive_packet, int socket, send_peer_t peer, 
-	time_t twin, time_t t_begin, int report_seq, int use_subdirs, char *time_extension, int compress, int do_xstat) {
+	time_t twin, time_t t_begin, int report_seq, int use_subdirs, char *time_extension, int compress) {
 common_flow_header_t	*nf_header;
 FlowSource_t			*fs;
 struct sockaddr_storage nf_sender;
@@ -397,11 +396,6 @@ srecord_t	*commbuff;
 		if ( !fs->nffile ) {
 			return;
 		}
-		if ( do_xstat ) {
-			fs->xstat = InitXStat(fs->nffile);
-			if ( !fs->xstat ) 
-				return;
-		}
 		// init vars
 		fs->bad_packets		= 0;
 		fs->first_seen      = 0xffffffffffffLL;
@@ -527,14 +521,6 @@ srecord_t	*commbuff;
 				nffile->stat_record->last_seen 	= fs->last_seen/1000;
 				nffile->stat_record->msec_last	= fs->last_seen - nffile->stat_record->last_seen*1000;
 
-				if ( fs->xstat ) {
-					if ( WriteExtraBlock(nffile, fs->xstat->block_header ) <= 0 ) 
-						LogError("Ident: %s, failed to write xstat buffer to disk: '%s'" , fs->Ident, strerror(errno));
-
-					ResetPortHistogram(fs->xstat->port_histogram);
-					ResetBppHistogram(fs->xstat->bpp_histogram);
-				}
-
 				// Flush Exporter Stat to file
 				FlushExporterStats(fs);
 				// Close file
@@ -583,10 +569,6 @@ srecord_t	*commbuff;
 						LogError("killed due to fatal error: ident: %s", fs->Ident);
 						break;
 					}
-					/* XXX needs fixing */
-					if ( fs->xstat ) {
-						// to be implemented
-					}
 				}
 
 				// Dump all extension maps and exporters to the buffer
@@ -768,7 +750,7 @@ FlowSource_t *fs;
 struct sigaction act;
 int		family, bufflen;
 time_t 	twin, t_start;
-int		sock, synctime, do_daemonize, expire, spec_time_extension, report_sequence, do_xstat;
+int		sock, synctime, do_daemonize, expire, spec_time_extension, report_sequence;
 int		subdir_index, sampling_rate, compress;
 int		c;
 #ifdef PCAP
@@ -798,7 +780,6 @@ char	*pcap_file;
 	expire			= 0;
 	sampling_rate	= 1;
 	compress		= NOT_COMPRESSED;
-	do_xstat		= 0;
 	memset((void *)&peer, 0, sizeof(send_peer_t));
 	peer.family		= AF_UNSPEC;
 	Ident			= "none";
@@ -842,9 +823,6 @@ char	*pcap_file;
 				printf("%s: Version: %s\n",argv[0], nfdump_version);
 				exit(0);
 				break;
-			case 'X':
-				do_xstat = 1;
-				break;
 			case 'D':
 				do_daemonize = 1;
 				break;
@@ -972,14 +950,21 @@ char	*pcap_file;
 				break;
 			case 'j':
 				if ( compress ) {
-					LogError("Use either -z for LZO or -j for BZ2 compression, but not both\n");
+					LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression\n");
 					exit(255);
 				}
 				compress = BZ2_COMPRESSED;
 				break;
+			case 'y':
+				if ( compress ) {
+					LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression\n");
+					exit(255);
+				}
+				compress = LZ4_COMPRESSED;
+				break;
 			case 'z':
 				if ( compress ) {
-					LogError("Use either -z for LZO or -j for BZ2 compression, but not both\n");
+					LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression\n");
 					exit(255);
 				}
 				compress = LZO_COMPRESSED;
@@ -1221,7 +1206,7 @@ char	*pcap_file;
 
 	LogInfo("Startup.");
 	run(receive_packet, sock, peer, twin, t_start, report_sequence, subdir_index, 
-		time_extension, compress, do_xstat);
+		time_extension, compress);
 	close(sock);
 	kill_launcher(launcher_pid);
 
