@@ -92,7 +92,6 @@
 #include "flist.h"
 #include "nfstatfile.h"
 #include "bookkeeper.h"
-#include "nfxstat.h"
 #include "collector.h"
 #include "exporter.h"
 #include "rbtree.h"
@@ -141,7 +140,7 @@ int verbose = 0;
 
 static const char *nfdump_version = VERSION;
 
-static int launcher_alive, periodic_trigger, launcher_pid;
+static int launcher_pid;
 static pthread_mutex_t  m_done  = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t terminate = PTHREAD_COND_INITIALIZER;
 static pthread_key_t buffer_key;
@@ -646,7 +645,6 @@ int err, done;
    		pthread_kill(args->parent, SIGUSR1);
 		pthread_exit((void *)args);
 	}
-	fs->xstat = NULL;
 
 	// init vars
 	fs->bad_packets		= 0;
@@ -764,9 +762,9 @@ int err, done;
 				UpdateBooks(fs->bookkeeper, t_start, 512*fstat.st_blocks);
 			}
 
-			LogInfo("Ident: '%s' Flows: %llu, Packets: %llu, Bytes: %llu, Max Flows: %u", 
+			LogInfo("Ident: '%s' Flows: %llu, Packets: %llu, Bytes: %llu, Max Flows: %u, Fragments: %u", 
 				fs->Ident, (unsigned long long)nffile->stat_record->numflows, (unsigned long long)nffile->stat_record->numpackets, 
-				(unsigned long long)nffile->stat_record->numbytes, NumFlows);
+				(unsigned long long)nffile->stat_record->numbytes, NumFlows, IPFragEntries());
 
 			// reset stats
 			fs->bad_packets = 0;
