@@ -53,7 +53,6 @@
 #include "nffile.h"
 #include "nfx.h"
 #include "bookkeeper.h"
-#include "nfxstat.h"
 #include "collector.h"
 #include "exporter.h"
 #include "nfnet.h"
@@ -443,6 +442,7 @@ struct order_mode_s {
 #define Default_PrintOrder 1		// order_mode[0].val
 static uint32_t	print_order_bits = 0;
 static uint32_t	PrintOrder 		 = 0;
+static uint32_t	GuessDirection 	 = 0;
 static uint32_t	NumStats 		 = 0;
 
 static uint64_t	byte_limit, packet_limit;
@@ -494,6 +494,12 @@ static uint64_t	flows_record(FlowTableRecord_t *record, int inout) {
 }
 
 static uint64_t	packets_record(FlowTableRecord_t *record, int inout) {
+		if ( GuessDirection && (record->flowrecord.srcport < record->flowrecord.dstport) ) {
+			if (inout == IN)
+				inout = OUT;
+			else if (inout == OUT)
+				inout = IN;
+		}
         if (inout == IN)
                 return record->counter[INPACKETS];
         else if (inout == OUT)
@@ -503,6 +509,12 @@ static uint64_t	packets_record(FlowTableRecord_t *record, int inout) {
 }
 
 static uint64_t	bytes_record(FlowTableRecord_t *record, int inout) {
+		if ( GuessDirection && (record->flowrecord.srcport < record->flowrecord.dstport) ) {
+			if (inout == IN)
+				inout = OUT;
+			else if (inout == OUT)
+				inout = IN;
+		}
         if (inout == IN)
                 return record->counter[INBYTES];
         else if (inout == OUT)
@@ -1436,6 +1448,7 @@ struct tm	*tbuff;
 
 } // End of PrintCvsStatLine
 
+
 void PrintFlowTable(printer_t print_record, uint32_t topN, int tag, int GuessDir, extension_map_list_t *extension_map_list) {
 hash_FlowTable *FlowTable;
 FlowTableRecord_t	*r;
@@ -1446,12 +1459,13 @@ uint32_t 			i;
 uint32_t			maxindex, c;
 char				*string;
 
+	GuessDirection = GuessDir;
 	FlowTable = GetFlowTable();
 	aggr_record_mask = GetMasterAggregateMask();
 	c = 0;
 	maxindex = FlowTable->NumRecords;
 	if ( PrintOrder ) {
-		// Sort according the date
+		// Sort according the requested order
 		SortList = (SortElement_t *)calloc(maxindex, sizeof(SortElement_t));
 
 		if ( !SortList ) {
@@ -1544,10 +1558,10 @@ char				*string;
 
 				// apply IP mask from aggregation, to provide a pretty output
 				if ( FlowTable->has_masks ) {
-					flow_record->v6.srcaddr[0] &= FlowTable->IPmask[0];
-					flow_record->v6.srcaddr[1] &= FlowTable->IPmask[1];
-					flow_record->v6.dstaddr[0] &= FlowTable->IPmask[2];
-					flow_record->v6.dstaddr[1] &= FlowTable->IPmask[3];
+					flow_record->V6.srcaddr[0] &= FlowTable->IPmask[0];
+					flow_record->V6.srcaddr[1] &= FlowTable->IPmask[1];
+					flow_record->V6.dstaddr[0] &= FlowTable->IPmask[2];
+					flow_record->V6.dstaddr[1] &= FlowTable->IPmask[3];
 				}
 
 				if ( aggr_record_mask ) {
@@ -1670,7 +1684,7 @@ uint32_t			maxindex, c;
 					else
 						printf("Top flows ordered by %s:\n", order_mode[order_index].string);
 				}
-				if ( !record_header ) 
+				if ( record_header ) 
 					printf("%s\n", record_header);
 			}
 			PrintSortedFlowcache(SortList, maxindex, topN, 0, print_record, tag, DESCENDING, extension_map_list);
@@ -1719,10 +1733,10 @@ int	i, max;
 		
 		// apply IP mask from aggregation, to provide a pretty output
 		if ( FlowTable->has_masks ) {
-			flow_record->v6.srcaddr[0] &= FlowTable->IPmask[0];
-			flow_record->v6.srcaddr[1] &= FlowTable->IPmask[1];
-			flow_record->v6.dstaddr[0] &= FlowTable->IPmask[2];
-			flow_record->v6.dstaddr[1] &= FlowTable->IPmask[3];
+			flow_record->V6.srcaddr[0] &= FlowTable->IPmask[0];
+			flow_record->V6.srcaddr[1] &= FlowTable->IPmask[1];
+			flow_record->V6.dstaddr[0] &= FlowTable->IPmask[2];
+			flow_record->V6.dstaddr[1] &= FlowTable->IPmask[3];
 		}
 
 		if ( FlowTable->apply_netbits ) {
@@ -1888,12 +1902,12 @@ uint64_t _tmp_ip[2];
 uint64_t _tmp_l;
 uint32_t _tmp;
 
-	_tmp_ip[0] = flow_record->v6.srcaddr[0];
-	_tmp_ip[1] = flow_record->v6.srcaddr[1];
-	flow_record->v6.srcaddr[0] = flow_record->v6.dstaddr[0];
-	flow_record->v6.srcaddr[1] = flow_record->v6.dstaddr[1];
-	flow_record->v6.dstaddr[0] = _tmp_ip[0];
-	flow_record->v6.dstaddr[1] = _tmp_ip[1];
+	_tmp_ip[0] = flow_record->V6.srcaddr[0];
+	_tmp_ip[1] = flow_record->V6.srcaddr[1];
+	flow_record->V6.srcaddr[0] = flow_record->V6.dstaddr[0];
+	flow_record->V6.srcaddr[1] = flow_record->V6.dstaddr[1];
+	flow_record->V6.dstaddr[0] = _tmp_ip[0];
+	flow_record->V6.dstaddr[1] = _tmp_ip[1];
 
 	_tmp = flow_record->srcport;
 	flow_record->srcport = flow_record->dstport;
@@ -1916,3 +1930,4 @@ uint32_t _tmp;
 	flow_record->out_bytes = _tmp_l;
 
 } // End of SwapFlow
+
