@@ -1,7 +1,6 @@
 /*
- *  Copyright (c) 2014, Peter Haag
- *  Copyright (c) 2009, Peter Haag
- *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
+ *  Copyright (c) 2009 - 2018, Peter Haag
+ *  Copyright (c) 2004 - 2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
  *  Redistribution and use in source and binary forms, with or without 
@@ -28,12 +27,6 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: haag $
- *
- *  $Id: profile.c 41 2009-12-31 14:46:28Z haag $
- *
- *  $LastChangedRevision: 41 $
- *      
  */
 
 #include "config.h"
@@ -62,12 +55,17 @@
 #include "nfdump.h"
 #include "nffile.h"
 #include "nfstatfile.h"
-#include "nfxstat.h"
 #include "flist.h"
 #include "util.h"
 #include "nftree.h"
 #include "profile.h"
 
+#ifdef HAVE_INFLUXDB
+#include <curl/curl.h>
+extern char influxdb_url[1024];
+static char influxdb_measurement[]="nfsen_stats";
+#endif
+
 /* imported vars */
 extern char yyerror_buff[256];
 extern uint32_t is_anonymized;
@@ -79,7 +77,7 @@ static unsigned int num_channels;
 static inline int AppendString(char *stack, char *string, size_t	*buff_size);
 
 static void SetupProfileChannels(char *profile_datadir, char *profile_statdir, profile_param_info_t *profile_param, 
-	int subdir_index, char *filterfile, char *filename, int verify_only, int compress, int do_xstat);
+	int subdir_index, char *filterfile, char *filename, int verify_only, int compress);
 
 profile_channel_info_t	*GetChannelInfoList(void) {
 	return profile_channels;
@@ -101,7 +99,7 @@ size_t len = strlen(string);
 } // End of AppendString
 
 unsigned int InitChannels(char *profile_datadir, char *profile_statdir, profile_param_info_t *profile_list, 
-	char *filterfile, char *filename, int subdir_index, int verify_only, int compress, int do_xstat) {
+	char *filterfile, char *filename, int subdir_index, int verify_only, int compress) {
 profile_param_info_t	*profile_param;
 
 	num_channels = 0;
@@ -111,7 +109,7 @@ profile_param_info_t	*profile_param;
 		profile_param->channelname, profile_param->profilename, profile_param->profilegroup, 
 		profile_param->channel_sourcelist);
 
-		SetupProfileChannels(profile_datadir, profile_statdir, profile_param, subdir_index, filterfile, filename, verify_only, compress, do_xstat);
+		SetupProfileChannels(profile_datadir, profile_statdir, profile_param, subdir_index, filterfile, filename, verify_only, compress);
 
 		profile_param = profile_param->next;
 	}
@@ -120,7 +118,7 @@ profile_param_info_t	*profile_param;
 } // End of InitChannels
 
 static void SetupProfileChannels(char *profile_datadir, char *profile_statdir, profile_param_info_t *profile_param, 
-	int subdir_index, char *filterfile, char *filename, int verify_only, int compress, int do_xstat ) {
+	int subdir_index, char *filterfile, char *filename, int verify_only, int compress ) {
 FilterEngine_data_t	*engine;
 struct 	stat stat_buf;
 char 	*p, *filter, *subdir, *wfile, *ofile, *rrdfile, *source_filter;
@@ -128,11 +126,9 @@ char	path[MAXPATHLEN];
 int		ffd, ret;
 size_t	filter_size;
 nffile_t *nffile;
-xstat_t	 *xstat;
 
 	ofile = wfile = NULL;
 	nffile = NULL;
-	xstat  = NULL;
 
 	/* 
 	 * Compile the complete filter:
@@ -313,12 +309,6 @@ xstat_t	 *xstat;
 		if ( !nffile ) {
 			return;
 		}
-		if ( do_xstat ) {
-			xstat = InitXStat(nffile);
-			if ( !xstat ) {
-				return;
-			}
-		}
 	} 
 
 	snprintf(path, MAXPATHLEN-1, "%s/%s/%s/%s.rrd", 
@@ -349,7 +339,6 @@ xstat_t	 *xstat;
 	profile_channels[num_channels].dirstat_path 			= strdup(path);
 	profile_channels[num_channels].type						= profile_param->profiletype;
 	profile_channels[num_channels].nffile					= nffile;
-	profile_channels[num_channels].xstat					= xstat;
 
 	memset((void *)&profile_channels[num_channels].stat_record, 0, sizeof(stat_record_t));
 
@@ -366,15 +355,9 @@ void CloseChannels (time_t tslot, int compress) {
 dirstat_t	*dirstat;
 struct stat fstat;
 unsigned int num;
-int ret, update_ok;
 
 	for ( num = 0; num < num_channels; num++ ) {
 		if ( profile_channels[num].ofile ) {
-			if ( profile_channels[num].xstat ) {
-				if ( WriteExtraBlock(profile_channels[num].nffile, profile_channels[num].xstat->block_header ) <= 0 ) {
-					LogError("Failed to write xstat buffer to disk: '%s'" , strerror(errno));
-				} 
-			}
 
 			if ( is_anonymized ) 
 				SetFlag(profile_channels[num].nffile->file_header->flags, FLAG_ANONYMIZED);
@@ -382,10 +365,7 @@ int ret, update_ok;
 			profile_channels[num].nffile = DisposeFile(profile_channels[num].nffile);
 
 			stat(profile_channels[num].ofile, &fstat);
-
-			ret = ReadStatInfo(profile_channels[num].dirstat_path, &dirstat, CREATE_AND_LOCK);
-			update_ok = ret == STATFILE_OK;
-
+			ReadStatInfo(profile_channels[num].dirstat_path, &dirstat, CREATE_AND_LOCK);
 	
 			if ( rename(profile_channels[num].ofile, profile_channels[num].wfile) < 0 ) {
 				LogError("Failed to rename file %s to %s: %s\n", 
@@ -402,6 +382,10 @@ int ret, update_ok;
 		}
 		if ( ((profile_channels[num].type & 0x8) == 0) && tslot > 0 ) {
 			UpdateRRD(tslot, &profile_channels[num]);
+#ifdef HAVE_INFLUXDB
+			if(strlen(influxdb_url) > 0)
+				UpdateInfluxDB(tslot, &profile_channels[num]);
+#endif
 		}
 	}
 
@@ -471,3 +455,87 @@ stat_record_t stat_record = channel->stat_record;
 	}
 
 } // End of UpdateRRD
+
+#ifdef HAVE_INFLUXDB
+static void influxdb_client_post(char *body) {
+CURLcode c;
+CURL *handle = curl_easy_init();
+	//curl -i -XPOST 'http://nbox-demo:8086/write?db=lucatest' --data-binary 'test,host=server01,region=us-west valueA=0.64 valueB=0.64 1434055562000000000'
+
+	curl_easy_setopt(handle, CURLOPT_URL, influxdb_url);
+	curl_easy_setopt(handle, CURLOPT_TIMEOUT, 5L);
+	curl_easy_setopt(handle, CURLOPT_CONNECTTIMEOUT, 3L);
+	curl_easy_setopt(handle, CURLOPT_POSTFIELDS, body);
+
+	c = curl_easy_perform(handle);
+
+	if (c == CURLE_OK) {
+		long status_code = 0;
+		if (curl_easy_getinfo(handle, CURLINFO_RESPONSE_CODE, &status_code) == CURLE_OK){
+			c = status_code;
+
+			if (status_code != 204){
+				LogError("INFLUXDB: %s Insert Error: HTTP %d\n", influxdb_url, status_code);
+			}
+		}
+	} else{
+		LogError("INFLUXDB: %s Curl Error: %s\n", influxdb_url, curl_easy_strerror(c));
+	}
+
+	curl_easy_cleanup(handle);
+}
+
+void UpdateInfluxDB( time_t tslot, profile_channel_info_t *channel ) {
+	char	buff[2048], *s;
+	int		len, buffsize;
+	stat_record_t stat_record = channel->stat_record;
+
+	char *groupname = strcmp(channel->group, ".")==0?"ROOT":channel->group;
+
+	buffsize = sizeof(buff);
+	s = buff;
+	len = snprintf(s, buffsize , "%s,channel=%s,profilegroup=%s,profile=%s ", influxdb_measurement, channel->channel, groupname, channel->profile);
+	buffsize -= len; s += len;
+	len = snprintf(s, buffsize , "flows=%llu", (long long unsigned)stat_record.numflows);
+	buffsize -= len; s += len;
+	len = snprintf(s, buffsize , ",flows_tcp=%llu", (long long unsigned)stat_record.numflows_tcp);
+	buffsize -= len; s += len;
+	len = snprintf(s, buffsize , ",flows_udp=%llu", (long long unsigned)stat_record.numflows_udp);
+	buffsize -= len; s += len;
+	len = snprintf(s, buffsize , ",flows_icmp=%llu", (long long unsigned)stat_record.numflows_icmp);
+	buffsize -= len; s += len;
+	len = snprintf(s, buffsize , ",flows_other=%llu", (long long unsigned)stat_record.numflows_other);
+	buffsize -= len; s += len;
+
+	len = snprintf(s, buffsize , ",packets=%llu", (long long unsigned)stat_record.numpackets);
+	buffsize -= len; s += len;
+	len = snprintf(s, buffsize , ",packets_tcp=%llu", (long long unsigned)stat_record.numpackets_tcp);
+	buffsize -= len; s += len;
+	len = snprintf(s, buffsize , ",packets_udp=%llu", (long long unsigned)stat_record.numpackets_udp);
+	buffsize -= len; s += len;
+	len = snprintf(s, buffsize , ",packets_icmp=%llu", (long long unsigned)stat_record.numpackets_icmp);
+	buffsize -= len; s += len;
+	len = snprintf(s, buffsize , ",packets_other=%llu", (long long unsigned)stat_record.numpackets_other);
+	buffsize -= len; s += len;
+
+	len = snprintf(s, buffsize , ",traffic=%llu", (long long unsigned)stat_record.numbytes);
+	buffsize -= len; s += len;
+	len = snprintf(s, buffsize , ",traffic_tcp=%llu", (long long unsigned)stat_record.numbytes_tcp);
+	buffsize -= len; s += len;
+	len = snprintf(s, buffsize , ",traffic_udp=%llu", (long long unsigned)stat_record.numbytes_udp);
+	buffsize -= len; s += len;
+	len = snprintf(s, buffsize , ",traffic_icmp=%llu", (long long unsigned)stat_record.numbytes_icmp);
+	buffsize -= len; s += len;
+	len = snprintf(s, buffsize , ",traffic_other=%llu", (long long unsigned)stat_record.numbytes_other);
+	buffsize -= len; s += len;
+	// timestamp in nanoseconds
+	len = snprintf(s, buffsize , " %llu000000000", (long long unsigned)tslot);
+	buffsize -= len; s += len;
+
+	influxdb_client_post(buff);
+
+	//DATA: test,host=server01,region=us-west valueA=0.64,valueB=0.64 1434055562000000000'
+} // End of UpdateInfluxDB
+
+#endif /* HAVE_INFLUXDB */
+
